package com.java8;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.util.Random;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import org.junit.jupiter.api.Test;

// CompletableFuture.thenApply() is inherited from CompletionStage.
// this method returns a new CompletionStage that when the stage completes normally is executed
// with this stage's result as the argument of the supplied function

class DemoTest {

  static final Random RANDOM = new Random();

  @Test
  void test_thenApplyDemo() throws ExecutionException, InterruptedException {
    CompletableFuture<String> c =
        CompletableFuture.supplyAsync(() -> "hello")
            .thenApply((data) -> {
              String result = "";
              try {
                result = delayedUpperCase(data) + " WORLD";
              } catch (InterruptedException e) {
                e.printStackTrace();
              }
              return result;
            });

    assertEquals("HELLO WORLD", c.get());
  }

  private String delayedUpperCase(String result) throws InterruptedException {
    Thread.sleep(1000);
    return result.toUpperCase();
  }

  @Test
  void test_thenApplyDemo1() throws ExecutionException, InterruptedException {
    int num1 = RANDOM.nextInt();
    int num2 = RANDOM.nextInt();

    CompletableFuture<String> c =
        CompletableFuture.supplyAsync(() -> multiply(num1, num2))
            .thenApply(this::print);

    String expected = "Result: " + (num1 * num2);
    assertEquals(expected, c.get());
  }

  private int multiply(int num1, int num2) {
    return num1 * num2;
  }

  private String print(int result) {
    return "Result: " + result;
  }
}
